@@ -3,6 +3,7 @@ import json
 import logging as log
 from functools import wraps
 from dateutil.parser import parse as parse_date
+import re
 
 class color():
     black = lambda x: '\033[30m' + str(x)+'\033[0;39m'
@@ -45,21 +46,26 @@ class UnknownField(Error):
     def __init__(self, field=None):
         super().__init__('unknown_field', field=field)
 
+class UniqueError(Error):
+    def __init__(self, field=None):
+        print(color.red(field))
+        super().__init__('unique_error', field=field)
+
 class InvalidValue(Error):
     def __init__(self, message=None, field=None):
         super().__init__('invalid_value', message=message, field=field)
 
 class InvalidDate(InvalidValue):
     def __init__(self, message=None, field=None):
-        super().__init__('Invalid date', message=message, field=field)
+        super().__init__(message=message, field=field)
 
 class InvalidInt(InvalidValue):
     def __init__(self, message=None, field=None):
-        super().__init__('Invalid int', message=message, field=field)
+        super().__init__(message=message, field=field)
 
 class InvalidFloat(InvalidValue):
     def __init__(self, message=None, field=None):
-        super().__init__('Invalid float', message=message, field=field)
+        super().__init__(message=message, field=field)
 
 class Clause:
     def __init__(self, fields, values, pattern='{name}', separator=', ', empty=''):
@@ -129,6 +135,7 @@ class Table(metaclass=MetaTable):
     id = 'id'
     fields = dict()
     joins = dict()
+    #order = {'field':'id', 'method':'desc'}
     get_db = lambda: None
     put_db = lambda db: None
     @classmethod
@@ -163,6 +170,8 @@ class Table(metaclass=MetaTable):
 
     @classmethod
     def where(cls, data, separator='AND'):
+        log.info(color.cyan('Where data %s'), data)
+
         if data is None:
             raise MissingInput()
 
@@ -170,6 +179,8 @@ class Table(metaclass=MetaTable):
         fields = []
         for field, config in cls.fields.items():
 
+            log.info(color.cyan('Parsing field %s %s'), field, config)
+
             if field in data:
 
                 value = data[field]
@@ -197,7 +208,6 @@ class Table(metaclass=MetaTable):
                     #criteria = "("+cls.name+'."'+column+"\" LIKE %s OR "+cls.name+'."'+column+"\" LIKE %s)"
                     criteria = cls.name+'."'+column+"\" ILIKE %s"
 
-
                 if 'array' in config and config['array']:
                     if not isinstance(value, list) and not isinstance(value, tuple):
                         raise InvalidValue('Value of '+field+' must be instance of list '+str(type(value))+' given', field)
@@ -205,27 +215,32 @@ class Table(metaclass=MetaTable):
                         values.append(cls.value(field, parse))
                         fields.append(criteria)
                 else:
-                    if (config['type'] == 'int' or
+                    if 'options' in config and (isinstance(value, list) or isinstance(value, tuple) or isinstance(value, set)):
+                        fields.append(cls.name+'."'+column+"\" IN ("+','.join(['%s'] * len(value))+")")
+                        for item in value:
+                            log.info(color.cyan('%s'), item)
+                            values.append(cls.value(field, item))
+                    elif (config['type'] == 'int' or
                         config['type'] == 'date' or
                         config['type'] == 'float') and \
                         (isinstance(value, list) or
                          isinstance(value, tuple) or
-                         isinstance(value, dict)) and \
-                        ('from' in value or
-                         'to' in value or
-                         'in' in value):
-                        if 'from' in value:
+                         isinstance(value, set) or
+                         isinstance(value, dict)):
+                        # ('from' in value or
+                        #  'to' in value or
+                        #  'in' in value):
+                        if isinstance(value, dict) and 'from' in value:
                             values.append(cls.value(field, value['from']))
                             fields.append(cls.name+'."'+column+"\">=%s")
-                        if 'to' in value:
+                        if isinstance(value, dict) and 'to' in value:
                             values.append(cls.value(field, value['to']))
                             fields.append(cls.name+'."'+column+"\"<=%s")
-                        if 'in' in value and (isinstance(value['in'], list) or isinstance(value['in'], tuple)) and len(value['in']):
-                            for item in value['in']:
+                        if (isinstance(value, list) or isinstance(value, tuple) or isinstance(value, set)) and len(value):
+                            for item in value:
                                 log.info(color.cyan('%s'), item)
                                 values.append(cls.value(field, item))
-                            fields.append(cls.name+'."'+column+"\" IN ("+','.join(['%s'] * len(value['in']))+")")
-
+                            fields.append(cls.name+'."'+column+"\" IN ("+','.join(['%s'] * len(value))+")")
                     else:
                         if config['type'] != 'json':
                             values.append(cls.value(field, value))
@@ -240,28 +255,30 @@ class Table(metaclass=MetaTable):
         return Clause(fields, values, separator=' '+separator+' ', empty='1=1')
 
     @classmethod
-    def order(cls, field, method=None, data=None):
-        if data:
-            if not 'field' in data:
-                return ''
+    def order(cls, field=None, method=None, data={}):
+        if 'field' in data:
+            field = data['field']
+        if 'method' in data:
+            method = data['method']
 
-            split = data['field'].split('.', 1)
-            if len(split) == 2:
-                data['key'] = split[1]
-                data['field'] = split[0]
-            elif len(split) > 2:
-                return ''
+        key = None
 
-            if data['field'] in cls.fields:
-                field = data['field']
-            else:
-                return ''
-        elif field:
-            if field not in cls.fields:
-                raise UnknownField()
-        else:
+        if not field:
             raise MissingField()
 
+        split = field.split('.', 1)
+        if len(split) == 2:
+            key = split[1]
+            field = split[0]
+        elif len(split) > 2:
+            return ''
+
+        if not field:
+            raise MissingField()
+
+        if field not in cls.fields:
+            raise UnknownField(field)
+
         config = cls.fields[field]
         if 'type' not in config:
             config['type'] = 'string'
@@ -269,34 +286,38 @@ class Table(metaclass=MetaTable):
         column = cls.name+'."'+(config['field'] if 'field' in config else field)+'"'
 
         if config['type'] == 'json':
-            if 'key' not in data:
+            if 'key' is None:
                 raise MissingField()
             if 'keys' not in config:
                 raise MissingConfig()
-            if data['key'] not in config['keys']:
+            if key not in config['keys']:
                 raise UnknownField()
-            column += "->'"+data['key']+"'"
+            column += "->'"+key+"'"
 
-        if data and 'method' in data:
-            method = data['method'].upper()
-            if method not in ['ASC', 'DESC']:
-                method = 'ASC'
-        else:
-            method = 'ASC'
+        if method is None:
+            method = ''
 
-        return column+' '+method
+        method = method.upper()
+
+        if method and method not in ['ASC', 'DESC']:
+            raise InvalidValue(method)
+
+        if method:
+            method = ' '+method
+
+        return column+method
 
     @classmethod
     def parse(cls, data, mode):
         if data is None:
             raise MissingInput()
 
-        log.info(color.cyan('In a parse %s'), cls.fields)
+        #log.info(color.cyan('In a parse %s'), cls.fields)
 
         values = []
         fields = []
         for field, config in cls.fields.items():
-            log.info(color.cyan('Parsing field %s'), field)
+            #log.info(color.cyan('Parsing field %s'), field)
             if mode in config and not config[mode]:
                 continue
 
@@ -430,20 +451,17 @@ class Table(metaclass=MetaTable):
         #return cls.type(**params)
     @classmethod
     def get(cls, id):
-        row = Row()
-        row.offset(cls.name, cls)
-        join = Join(cls, row)
+        join = Join(cls)
         try:
             db = cls.get_db()
             cursor = db.cursor()
-            cursor.execute(f"""SELECT {select(cls.select(), join.select())}
+            cursor.execute(f"""SELECT {join.select()}
                              FROM {cls}
                              {join}
                              WHERE {cls('id')}=%s""",
                              (id,))
-            row.data(cursor.fetchone())
-            result = cls.create(row(cls.name))
-            join.set(result)
+            join.row.data(cursor.fetchone())
+            result = join.create()
             return result
         except Exception as error:
             raise error
@@ -452,14 +470,8 @@ class Table(metaclass=MetaTable):
             cls.put_db(db)
 
     @classmethod
-    def all(cls, filter={}, order={}, search={}):
-        filter = cls.where(filter)
-        search = cls.where(search, separator='OR')
-
-        row = Row()
-        row.offset(cls.name, cls)
-        join = Join(cls, row)
-        row.offset('total')
+    def all(cls, filter={}, order={}, search={}, limit=None):
+        join = Join(cls, filter, search)
 
         result = []
 
@@ -467,18 +479,18 @@ class Table(metaclass=MetaTable):
             db = cls.get_db()
             cursor = db.cursor()
             cursor.execute(*debug(f"""SELECT
-                           {select(cls.select(), join.select())}
+                           {join.select()}
                            FROM {cls}
                            {join}
-                           WHERE ({filter.fields()}) AND ({search.fields()})
-                           ORDER BY {cls.order('id', 'desc', order)}""",
-                           filter.values()+search.values()))
-            log.debug(color.cyan('Total fetched %s'))
+                           WHERE {join.fields()}
+                           ORDER BY {cls.order('id', 'desc', order)}
+                           {f'LIMIT {int(limit)}' if limit else ''}""",
+                           join.values()))
+            log.debug(color.cyan('Total fetched %s'), cursor.rowcount)
             while True:
                 try:
-                    row.data(cursor.fetchone())
-                    item = cls.create(row(cls.name))
-                    join.set(item)
+                    join.row.data(cursor.fetchone())
+                    item = join.create()
                     result.append(item)
                 except TypeError:
                     break
@@ -492,38 +504,35 @@ class Table(metaclass=MetaTable):
 
     @classmethod
     def filter(cls, page=1, limit=100, filter={}, order={}, search={}):
-        filter = cls.where(filter)
-        search = cls.where(search, separator='OR')
+        join = Join(cls, filter, search)
+        join.row.offset('total')
+
         limit = min(limit, 100)
         offset = (page-1)*limit
 
-        row = Row()
-        row.offset(cls.name, cls)
-        join = Join(cls, row)
-        row.offset('total')
-
         result = Result()
 
+        db = None
+
         try:
             db = cls.get_db()
             cursor = db.cursor()
             cursor.execute(*debug(f"""SELECT
-                           {select(cls.select(), join.select())},
+                           {join.select()},
                            COUNT(*) OVER()
                            FROM {cls}
                            {join}
-                           WHERE ({filter.fields()}) AND ({search.fields()})
-                           ORDER BY {cls.order('id', 'desc', order)}
+                           WHERE {join.fields()}
+                           ORDER BY {join.order('id', 'desc', order)}
                            LIMIT %s OFFSET %s""",
-                           filter.values()+search.values()+[limit, offset]))
+                           join.values()+[limit, offset]))
+            log.debug(color.cyan('Total fetched %s'), cursor.rowcount)
             while True:
                 try:
-                    row.data(cursor.fetchone())
+                    join.row.data(cursor.fetchone())
                     if result.total is None:
-                        result.total = row('total')
-                        print('Total', result.total)
-                    item = cls.create(row(cls.name))
-                    join.set(item)
+                        result.total = join.row('total')
+                    item = join.create()
                     result.add(item)
                 except TypeError:
                     break
@@ -541,27 +550,24 @@ class Table(metaclass=MetaTable):
     @classmethod
     def save(cls, id, data, filter={}):
         filter = cls.where(filter)
+        join = Join(cls)
         update = cls.update(data)
-        row = Row()
-        row.offset(cls.name, cls)
-        join = Join(cls, row)
         try:
             db = cls.get_db()
             cursor = db.cursor()
-            cursor.execute(*debug(f"""WITH {cls.name} AS (
+            cursor.execute(*debug(f"""WITH "{cls.name}" AS (
                                         UPDATE {cls}
                                         SET {update.fields()}
                                         WHERE {cls('id')}=%s AND {filter.fields()}
                                         RETURNING {cls.select()}
                                     )
-                                    SELECT {select(f'{cls.name}.*', join.select())}
-                                    FROM {cls.name}
+                                    SELECT {join.select()}
+                                    FROM "{cls.name}"
                                     {join}
                                     """,
                                 update.values(id)+filter.values()))
-            row.data(cursor.fetchone())
-            result = cls.create(row(cls.name))
-            join.set(result)
+            join.row.data(cursor.fetchone())
+            result = join.create()
         except Exception as error:
             raise error
         finally:
@@ -572,29 +578,35 @@ class Table(metaclass=MetaTable):
 
     @classmethod
     def add(cls, data):
-        row = Row()
-        row.offset(cls.name, cls)
-        join = Join(cls, row)
+        join = Join(cls)
         insert = cls.insert(data)
         try:
             db = cls.get_db()
             cursor = db.cursor()
-            cursor.execute(*debug(f"""WITH {cls.name} AS (
+            cursor.execute(*debug(f"""WITH "{cls.name}" AS (
                                         INSERT INTO {cls}
                                         ({insert.fields()})
                                         VALUES ({insert.fields('%s')})
                                         RETURNING {cls.select()}
                                     )
-                                    SELECT {select(f'{cls.name}.*', join.select())}
-                                    FROM {cls.name}
+                                    SELECT {join.select()}
+                                    FROM "{cls.name}"
                                     {join}
                                     """,
                                 insert.values()))
+            join.row.data(cursor.fetchone())
+            result = join.create()
 
-            row.data(cursor.fetchone())
-            result = cls.create(row(cls.name))
-            join.set(result)
         except Exception as error:
+            match = re.search(r''+cls.name+'_unique_(.*?)_index', str(error))
+            print(match)
+            if match is not None and match.lastindex > 0:
+                index = match.group(1)
+                if index in cls.fields:
+                    raise UniqueError(index)
+                for field, config in cls.fields:
+                    if 'field' in config and config['field'] == index:
+                        raise UniqueError(field)
             raise error
         finally:
             db.commit()
@@ -603,13 +615,14 @@ class Table(metaclass=MetaTable):
         return result
 
     @classmethod
-    def delete(cls, id):
+    def delete(cls, id, filter={}):
+        filter = cls.where(filter)
         try:
             db = cls.get_db()
             cursor = db.cursor()
             cursor.execute(*debug(f"""DELETE FROM {cls}
-                                    WHERE {cls('id')}=%s""",
-                                (id,)))
+                                    WHERE {cls('id')}=%s AND {filter.fields()}""",
+                                (id,)+filter.values()))
             return bool(cursor.rowcount)
         except Exception as error:
             raise error
@@ -643,16 +656,52 @@ class Row:
         return self.get(name)
 
 class Join():
-    def __init__(self, table, row):
+    def __init__(self, table, filter={}, search={}):
         self.table = table
-        self.row = row
+
+        self.row = Row()
+        self.row.offset(self.table.name, self.table)
         if self.table.joins:
             for join in self.table.joins.values():
                 self.row.offset(join['table'].name, join['table'])
+
+        self.searchs = {}
+        self.filters = {}
+        if filter:
+            self.filters[self.table.name] = self.table.where(filter)
+            if self.table.joins:
+                for key, value in self.table.joins.items():
+                    if key in filter:
+                        self.filters[key] = value['table'].where(filter[key])
+
+        if search:
+            self.searchs[self.table.name] = self.table.where(search, separator='OR')
+            if self.table.joins:
+                for key, value in self.table.joins.items():
+                    if key in search:
+                        self.searchs[key] = value['table'].where(search[key], separator='OR')
+
     def select(self):
         result = ''
         if self.table.joins:
             result = ','.join([join['table'].select() for join in self.table.joins.values()])
+        return select(self.table.select(), result)
+
+    def fields(self):
+        search = ' OR '.join(where.fields() for where in self.searchs.values())
+        filter = ' AND '.join(where.fields() for where in self.filters.values())
+        if not search:
+            search = '1=1'
+        if not filter:
+            filter = '1=1'
+        return '('+search+ ') AND ('+filter+')'
+
+    def values(self):
+        result = []
+        for where in self.searchs.values():
+            result.extend(where.values())
+        for where in self.filters.values():
+            result.extend(where.values())
         return result
 
     def clause(self, name):
@@ -664,19 +713,40 @@ class Join():
             return '\n'.join([self.clause(join) for join in self.table.joins.keys()])
         return ''
 
-    def set(self, item):
+    def create(self):
+        item = self.table.create(self.row(self.table.name))
         if self.table.joins:
             for name, join in self.table.joins.items():
                 setattr(item, name, join['table'].create(self.row(join['table'].name)))
+        return item
+
+    def order(self, field, method, order={}):
+        if 'field' in order:
+            field = order['field']
+        if 'method' in order:
+            method = order['method']
+
+        order = None
+        if self.table.joins:
+            for name, join in self.table.joins.items():
+                if field.startswith(name+'.'):
+                    try:
+                        order = join['table'].order(data={'field':field[len(name)+1:], 'method':method})
+                    except Exception:
+                        pass
+        if not order:
+            return self.table.order(field, method)
+        return order
 
 def select(*args):
-    return ','.join([item for item in args if str(item).strip()!=''])
+    return ','.join([item for item in args if str(item).strip() != ''])
 
 class Result():
     def __init__(self, total=None):
         self.total = total
         self.items = []
     def add(self, item):
+        #log.info('Adding %s', item)
         self.items.append(item)
 
 def debug(query, params):
@@ -716,6 +786,9 @@ def debug(query, params):
 
     if query_debug.count('%s') != len(params_debug):
         log.error(color.red('Query contains %s params while %s provided'), query_debug.count('%s'), len(params_debug))
-    log.debug(query_debug % params_debug)
+        log.debug(query_debug)
+        log.debug(params_debug)
+    else:
+        log.debug(query_debug % params_debug)
 
     return (query, params)
